// g++ -O2 -Wall -o perfs_boost.x -lstdc++ -lboost_signals perfs_boost.cpp
#include <iostream>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <sys/time.h>
#include <time.h>
#include <boost/signals/signal0.hpp>

typedef boost::signal0<void> SigType ;

struct SlotType
{
  void operator() ()
  { }
} ;

SlotType* slot_generator()
{ return new SlotType ; }

void slot_delete(SlotType* sl)
{ delete sl ; }

SigType* sig_generator()
{ return new SigType ; }

void sig_delete(SigType* sig)
{ delete sig ; }

struct slot_connector
{
  slot_connector (SigType* sig) : signal(sig) { }
  void operator() (SlotType* sl)
  {
    signal->connect(*sl) ;
  }
  SigType* signal ;
} ;

struct sig_connector
{
  sig_connector (std::vector<SlotType*>::const_iterator b,
                 std::vector<SlotType*>::const_iterator e) :
    slot_begin(b),
    slot_end(e)
  { }
  void operator() (SigType* sig)
  {
    std::for_each(slot_begin, slot_end, slot_connector(sig)) ;
  }
  std::vector<SlotType*>::const_iterator slot_begin ;
  std::vector<SlotType*>::const_iterator slot_end ;
} ;

inline void sig_emitter(SigType* sig)
{
  sig->operator()() ;
}

struct timeval t_start ;
struct timeval t_stop ;

void init_chrono()
{
  gettimeofday(&t_start, 0) ;
}

double elapsed()
{
  gettimeofday(&t_stop, 0) ;
  return (double)((t_stop.tv_sec - t_start.tv_sec)) +
             ((double)(t_stop.tv_usec - t_start.tv_usec))/1000000.0 ;
}

int main (int argc, char* argv[])
{
  unsigned nb = 5 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 5 ;

  double dnb = nb ;

  std::vector<SigType*> signals(nb) ;
  std::vector<SlotType*> targets(nb) ;

  std::cout << "Generating " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::generate(signals.begin(), signals.end(), sig_generator) ;
  double e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << nb/e << " nb/sec\n" ;

  std::generate(targets.begin(), targets.end(), slot_generator) ;

  std::cout << "Connecting " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::for_each(signals.begin(), signals.end(),
    sig_connector(targets.begin(), targets.end())) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::cout << "Emitting   " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::for_each(signals.begin(), signals.end(), sig_emitter) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::cout << "Deleting   " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::for_each(signals.begin(), signals.end(), sig_delete) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::for_each(targets.begin(), targets.end(), slot_delete) ;

  std::cout << "---\n" ;

  return 0 ;
}
