// moc perfs_qt.h > moc_perfs_qt.cpp
// g++ -O2 -Wall -o perfs_qt.x perfs_qt.cpp moc_perfs_qt.cpp -lstdc++ -I$QTDIR/include -L$QTDIR/lib -lqt-mt
#include <qobject.h>
#include <iostream>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <sys/time.h>
#include <time.h>
#include "perfs_qt.h"

Sig0::Sig0() :
  QObject(0,0)
{ }

inline void Sig0::emitter()
{ emit sig() ; }

Slot0::Slot0() :
  QObject(0,0)
{ }

void Slot0::slot()
{ }

Slot0* slot_generator()
{ return new Slot0 ; }

void slot_delete(Slot0* s)
{ delete s ; }

Sig0* sig_generator()
{ return new Sig0 ; }

void sig_delete(Sig0* sig)
{ delete sig ; }

struct slot_connector
{
  slot_connector (Sig0* sig) : signal(sig) { }
  void operator() (Slot0* sl)
  {
    sl->connect (signal, SIGNAL(sig()),
                 sl,     SLOT(slot())) ;
  }
  Sig0* signal ;
} ;

struct sig_connector
{
  sig_connector (std::vector<Slot0*>::const_iterator b,
                 std::vector<Slot0*>::const_iterator e) :
    slot_begin(b),
    slot_end(e)
  { }
  void operator() (Sig0* sig)
  {
    std::for_each(slot_begin, slot_end, slot_connector(sig)) ;
  }
  std::vector<Slot0*>::const_iterator slot_begin ;
  std::vector<Slot0*>::const_iterator slot_end ;
} ;

inline void sig_emitter(Sig0* sig)
{
  sig->emitter() ;
}

struct timeval t_start ;
struct timeval t_stop ;

void init_chrono()
{
  gettimeofday(&t_start, 0) ;
}

double elapsed()
{
  gettimeofday(&t_stop, 0) ;
  return (double)((t_stop.tv_sec - t_start.tv_sec)) +
             ((double)(t_stop.tv_usec - t_start.tv_usec))/1000000.0 ;
}

int main (int argc, char* argv[])
{
  unsigned nb = 5 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 5 ;

  double dnb = nb ;

  std::vector<Sig0*> _signals(nb) ;
  std::vector<Slot0*> targets(nb) ;

  std::cout << "Generating " << std::setw(5) << nb << " _signals... " ;
  init_chrono() ;
  std::generate(_signals.begin(), _signals.end(), sig_generator) ;
  double e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << nb/e << " nb/sec\n" ;

  std::generate(targets.begin(), targets.end(), slot_generator) ;

  std::cout << "Connecting " << std::setw(5) << nb << " _signals... " << std::flush ;
  init_chrono() ;
  std::for_each(_signals.begin(), _signals.end(),
    sig_connector(targets.begin(), targets.end())) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::cout << "Emitting   " << std::setw(5) << nb << " _signals... " << std::flush ;
  init_chrono() ;
  std::for_each(_signals.begin(), _signals.end(), sig_emitter) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::cout << "Deleting   " << std::setw(5) << nb << " _signals... " << std::flush ;
  init_chrono() ;
  std::for_each(_signals.begin(), _signals.end(), sig_delete) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::for_each(targets.begin(), targets.end(), slot_delete) ;

  std::cout << "---\n" ;

  return 0 ;
}
