// g++ -O2 -Wall -o perfs.x perfs.cpp -lstdc++
#include <iostream>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <sys/time.h>
#include <time.h>
#include "../slotsig/slotsig_0.h"

typedef SlotSig::Sig0<void> SigType ;

struct C : public SlotSig::SlotsSetBase
{
  C() : SlotSig::SlotsSetBase() { }
  void f() { }
} ;

C* c_generator()
{ return new C ; }

void c_delete(C* c)
{ delete c ; }

SigType* sig_generator()
{ return new SigType ; }

void sig_delete(SigType* sig)
{ delete sig ; }

struct c_connector
{
  c_connector (SigType* sig) : signal(sig) { }
  void operator() (C* c)
  {
    signal->connect (c, &C::f) ;
  }
  SigType* signal ;
} ;

struct sig_connector
{
  sig_connector (std::vector<C*>::const_iterator b,
                 std::vector<C*>::const_iterator e) :
    c_begin(b),
    c_end(e)
  { }
  void operator() (SigType* sig)
  {
    std::for_each(c_begin, c_end, c_connector(sig)) ;
  }
  std::vector<C*>::const_iterator c_begin ;
  std::vector<C*>::const_iterator c_end ;
} ;

inline void sig_emitter(SigType* sig)
{
  sig->emit() ;
}

struct timeval t_start ;
struct timeval t_stop ;

void init_chrono()
{
  gettimeofday(&t_start, 0) ;
}

double elapsed()
{
  gettimeofday(&t_stop, 0) ;
  return (double)((t_stop.tv_sec - t_start.tv_sec)) +
             ((double)(t_stop.tv_usec - t_start.tv_usec))/1000000.0 ;
}

int main (int argc, char* argv[])
{
  unsigned nb = 5 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 5 ;

  double dnb = nb ;

  std::vector<SigType*> signals(nb) ;
  std::vector<C*> targets(nb) ;

  std::cout << "Generating " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::generate(signals.begin(), signals.end(), sig_generator) ;
  double e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << nb/e << " nb/sec\n" ;

  std::generate(targets.begin(), targets.end(), c_generator) ;

  std::cout << "Connecting " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::for_each(signals.begin(), signals.end(),
    sig_connector(targets.begin(), targets.end())) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::cout << "Emitting   " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::for_each(signals.begin(), signals.end(), sig_emitter) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::cout << "Deleting   " << std::setw(5) << nb << " signals... " << std::flush ;
  init_chrono() ;
  std::for_each(signals.begin(), signals.end(), sig_delete) ;
  e = elapsed() ;
  std::cout << std::setprecision(15) << std::setw(20) << e << "sec, "
            << std::setprecision(15) << std::setw(20) << (dnb*dnb)/e << " nb/sec\n" ;

  std::for_each(targets.begin(), targets.end(), c_delete) ;

  std::cout << "---\n" ;

  return 0 ;
}
