
/*
 * slotsig_1.h header file, auto-generated by generate_slotsig.py
 *
 * Part of the SlogSig project, an attempt to provide a type-safe
 * signals/slots mechanism for the C++ language.
 * This file provides support for signals and slots using 1 parameter(s)
 *
 * Copyright (C) 2003 Yves Bailly <kafka_AT_kafka-fr_DOT_net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

#ifndef __SLOTSIG_1_H__
#define __SLOTSIG_1_H__ 1

#include <set>
#include <functional>
#include <algorithm>

#include "slotsig_bases.h"

namespace SlotSig {

template <typename _RETURN_TYPE, typename _ARG0_TYPE>
class Slot1 : public SlotBase
{ public:
    virtual _RETURN_TYPE operator() (_ARG0_TYPE arg0) const = 0 ;
    virtual bool operator== (Slot1<_RETURN_TYPE, _ARG0_TYPE>* sl) const = 0 ;
} ; // class Slot1<>

template <class _C, typename _RETURN_TYPE, typename _METHOD_TYPE, typename _ARG0_TYPE>
class SlotInClass1 : public Slot1<_RETURN_TYPE, _ARG0_TYPE>
{
  public:
    SlotInClass1 (_C* c, _METHOD_TYPE m) :
      Slot1<_RETURN_TYPE, _ARG0_TYPE>(),
      in_class(c),
      method(m)
    { }

    virtual _RETURN_TYPE operator()(_ARG0_TYPE arg0) const
    {
      return (in_class->*method)(arg0) ;
    }

    virtual bool operator== (Slot1<_RETURN_TYPE , _ARG0_TYPE>* sl) const
    {
      SlotInClass1<_C, _RETURN_TYPE, _METHOD_TYPE, _ARG0_TYPE>* sic =
        dynamic_cast<SlotInClass1<_C, _RETURN_TYPE, _METHOD_TYPE, _ARG0_TYPE>*>(sl) ;
      if ( sic == 0 )
        return 0 ;
      return ( (in_class == sic->in_class) &&
               (method == sic->method) ) ;
    }

  protected:
    _C* in_class ;
    _METHOD_TYPE method ;
} ; // class SlotInClass1

template <typename _RETURN_TYPE, typename _ARG0_TYPE>
struct equal_slot1
{
  equal_slot1 (Slot1<_RETURN_TYPE, _ARG0_TYPE>* sl) :
    slot(sl)
  { }

  bool operator() (Slot1<_RETURN_TYPE, _ARG0_TYPE>* sl)
  {
    return (*slot == sl) ;
  }

  Slot1<_RETURN_TYPE, _ARG0_TYPE>* slot ;
} ;

template <typename _RETURN_TYPE, typename _ARG0_TYPE>
class Sig1 : public SignalBase
{
  public:
    typedef _RETURN_TYPE (*global_slot_t) (_ARG0_TYPE arg0) ;

    Sig1() :
      SignalBase(),
      global_slots(),
      method_slots()
    { }

    ~Sig1()
    {
      register const typename methods_slots_set::const_iterator end =
        method_slots.end() ;
      register typename methods_slots_set::const_iterator iter =
        method_slots.begin() ;
      while ( iter != end )
        delete *(iter++) ;
    }

    void connect (global_slot_t f)
    {
      if ( f == 0 )
        return ;
      global_slots.insert(f) ;
    }

    template <class _C>
    void connect (_C* in_class,
                  _RETURN_TYPE (_C::*method)(_ARG0_TYPE))
    {
      connectGen(in_class, method) ;
    }

    template <class _C>
    void connect (_C* in_class,
                  _RETURN_TYPE (_C::*method)(_ARG0_TYPE) const)
    {
      connectGen(in_class, method) ;
    }

    template <class _C, class _C_PARENT>
    void connect (_C* in_class,
                  _RETURN_TYPE (_C_PARENT::*methode)(_ARG0_TYPE))
    {
      _C_PARENT* parent_class = dynamic_cast<_C_PARENT*>(in_class) ;
      if ( parent_class != 0 )
        connect (parent_class, methode) ;
    }

    template <class _C, class _C_PARENT>
    void connect (_C* in_class,
                  _RETURN_TYPE (_C_PARENT::*methode)(_ARG0_TYPE) const)
    {
      _C_PARENT* parent_class = dynamic_cast<_C_PARENT*>(in_class) ;
      if ( parent_class != 0 )
        connectGen (parent_class, methode) ;
    }

    void disconnect (global_slot_t f)
    {
      global_slots.erase(f) ;
    }

    template <class _C>
    void disconnect (_C* in_class,
                     _RETURN_TYPE (_C::*methode)(_ARG0_TYPE))
    {
      disconnectGen(in_class, methode) ;
    }

    template <class _C>
    void disconnect (_C* in_class,
                     _RETURN_TYPE (_C::*methode)(_ARG0_TYPE) const)
    {
      disconnectGen(in_class, methode) ;
    }

    template <class _C, class _C_PARENT>
    void disconnect (_C* in_class,
                     _RETURN_TYPE (_C_PARENT::*methode)(_ARG0_TYPE))
    { _C_PARENT* parent_class = dynamic_cast<_C_PARENT*>(in_class) ;
      if ( parent_class != 0 )
        disconnect (parent_class, methode) ;
    }

    template <class _C, class _C_PARENT>
    void disconnect (_C* in_class,
                     _RETURN_TYPE (_C_PARENT::*methode)(_ARG0_TYPE) const)
    { _C_PARENT* parent_class = dynamic_cast<_C_PARENT*>(in_class) ;
      if ( parent_class != 0 )
        disconnectGen (parent_class, methode) ;
    }

    void emit(_ARG0_TYPE arg0) const
    {
      register const typename global_slots_set::const_iterator end =
        global_slots.end() ;
      register typename global_slots_set::const_iterator iter =
        global_slots.begin() ;
      while ( iter != end )
        (*(iter++))(arg0) ;
      register const typename methods_slots_set::const_iterator end2 =
        method_slots.end() ;
      register typename methods_slots_set::const_iterator iter2 =
        method_slots.begin() ;
      while ( iter2 != end2 )
        (*(*(iter2++)))(arg0) ;
    }

    template <class _ACCUMULATOR>
    void emit(_ACCUMULATOR& accum, _ARG0_TYPE arg0) const
    {
      register const typename global_slots_set::const_iterator end =
        global_slots.end() ;
      register typename global_slots_set::const_iterator iter =
        global_slots.begin() ;
      while ( iter != end )
        accum((*(iter++))(arg0)) ;
      register const typename methods_slots_set::const_iterator end2 =
        method_slots.end() ;
      register typename methods_slots_set::const_iterator iter2 =
        method_slots.begin() ;
      while ( iter2 != end2 )
        accum((*(*(iter2++)))(arg0)) ;
    }

  protected:
    virtual void disconnectSlot(SlotBase*& sl)
    {
      Slot1<_RETURN_TYPE, _ARG0_TYPE>* slot =
        dynamic_cast<Slot1<_RETURN_TYPE, _ARG0_TYPE>*>(sl) ;
      if ( slot == 0 )
        return ;
      method_slots.erase(slot) ;
      delete sl ;
      sl = 0 ;
    }

  private:
    template <class _C, typename _METHOD_TYPE>
    void connectGen (_C* in_class,
                     _METHOD_TYPE method)
    {
      if ( in_class == 0 )
        return ;
      SlotInClass1<_C, _RETURN_TYPE, _METHOD_TYPE, _ARG0_TYPE>* slot =
        new SlotInClass1<_C, _RETURN_TYPE, _METHOD_TYPE, _ARG0_TYPE>(in_class, method) ;
      typename methods_slots_set::const_iterator iter =
        std::find_if(method_slots.begin(), method_slots.end(),
          equal_slot1<_RETURN_TYPE, _ARG0_TYPE>(slot)) ;
      if ( iter != method_slots.end() )
        delete slot ;
      else
      {
        method_slots.insert(slot) ;
        connectTo(in_class, slot) ;
      }
    }

    template <class _C, typename _METHOD_TYPE>
    void disconnectGen (_C* in_class,
                        _METHOD_TYPE method)
    {
      if ( in_class == 0 )
        return ;
      SlotInClass1<_C, _RETURN_TYPE, _METHOD_TYPE, _ARG0_TYPE>
        slot(in_class, method) ;
      typename methods_slots_set::const_iterator iter =
        std::find_if(method_slots.begin(), method_slots.end(),
          equal_slot1<_RETURN_TYPE, _ARG0_TYPE>(&slot)) ;
      if ( iter != method_slots.end() )
      { disconnectTo(in_class, *iter) ; }
    }

    typedef std::set<global_slot_t> global_slots_set ;
    global_slots_set global_slots ;

    typedef std::set<Slot1<_RETURN_TYPE, _ARG0_TYPE>*> methods_slots_set ;
    methods_slots_set method_slots ;
} ; // class Sig1<>

} // namespace SlotSig

#endif // __SLOTSIG_1_H__
