
/*
 * test_3.cpp test file, auto-generated by generate_slotsig.py
 *
 * Part of the SlogSig project, an attempt to provide a type-safe
 * signals/slots mechanism for the C++ language.
 * This file is a test program for signals and slots using 3 parameter(s)
 *
 * Copyright (C) 2003 Yves Bailly <kafka_AT_kafka-fr_DOT_net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

#include <iostream>
#include <string>
#include "../slotsig/slotsig_3.h"

class C1 : public SlotSig::SlotsSetBase
{
  public:
    C1() : SlotSig::SlotsSetBase() { }

    std::string m1(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::m1 (" << s << ")\n" ;
      return "C1::m1" ;
    }

    std::string m2(const std::string& s0, const std::string& s1, const std::string& s2) const
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::m2 (" << s << ")\n" ;
      return "C1::m2" ;
    }

    virtual std::string vm1(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::vm1 (" << s << ")\n" ;
      return "C1::vm1" ;
    }

    virtual std::string vm2(const std::string& s0, const std::string& s1, const std::string& s2) const
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::vm2 (" << s << ")\n" ;
      return "C1::vm2" ;
    }
} ; // C1

class C2 : public C1
{
  public:
    C2() : C1() { }

    virtual std::string vm1(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C2::vm1 (" << s << ")\n" ;
      return "C2::vm1" ;
    }

    virtual std::string vm2(const std::string& s0, const std::string& s1, const std::string& s2) const
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C2::vm2 (" << s << ")\n" ;
      return "C2::vm2" ;
    }
} ; // C2

std::string global_slot(const std::string& s0, const std::string& s1, const std::string& s2)
{
  std::string s = s0 + ", " + s1 + ", " + s2 ;
  std::cout << "::global_slot (" << s << ")\n" ;
  return "::global_slot" ;
}

struct Accum
{
  Accum() : str("") { }

  void operator() (const std::string& s)
  { str += (s + " ") ; }

  std::string str ;
} ; // Accum

int main()
{
  std::cout << "--- Test program START ---\n" ;

  C1* c1 = new C1 ;
  C2* c2 = new C2 ;
  typedef SlotSig::Sig3<std::string, const std::string&, const std::string&, const std::string&> SigType ;
  SigType* sig1 = new SigType ;
  SigType* sig2 = new SigType ;
  Accum accum ;

  std::cout << "--- Connecting all methods from c1 to sig1...\n" ;
  sig1->connect (global_slot) ;
  sig1->connect (c1, &C1::m1) ;
  sig1->connect (c1, &C1::m2) ;
  sig1->connect (c1, &C1::vm1) ;
  sig1->connect (c1, &C1::vm2) ;
  std::cout << "--- Emitting sig1 #1...\n" ;
  sig1->emit("e1_0", "e1_1", "e1_2") ;
  std::cout << "--- Emitting sig1 #2 with accumulator...\n" ;
  accum.str = "" ;
  sig1->emit(accum, "e2_0", "e2_1", "e2_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Re-connecting all methods from c1 to sig1...\n" ;
  sig1->connect (global_slot) ;
  sig1->connect (c1, &C1::m1) ;
  sig1->connect (c1, &C1::m2) ;
  sig1->connect (c1, &C1::vm1) ;
  sig1->connect (c1, &C1::vm2) ;
  std::cout << "--- Emitting sig1 #3...\n" ;
  sig1->emit("e3_0", "e3_1", "e3_2") ;
  std::cout << "--- Emitting sig1 #4 with accumulator...\n" ;
  accum.str = "" ;
  sig1->emit(accum, "e4_0", "e4_1", "e4_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Connecting all methods from c2 to sig1...\n" ;
  sig1->connect (c2, &C2::m1) ;
  sig1->connect (c2, &C2::m2) ;
  sig1->connect (c2, &C2::vm1) ;
  sig1->connect (c2, &C2::vm2) ;
  std::cout << "--- Emitting sig1 #5...\n" ;
  sig1->emit("e5_0", "e5_1", "e5_2") ;
  std::cout << "--- Emitting sig1 #6 with accumulator...\n" ;
  accum.str = "" ;
  sig1->emit(accum, "e6_0", "e6_1", "e6_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Connecting all methods from c1 and c2 to sig2...\n" ;
  sig2->connect (global_slot) ;
  sig2->connect (c1, &C1::m1) ;
  sig2->connect (c1, &C1::m2) ;
  sig2->connect (c1, &C1::vm1) ;
  sig2->connect (c1, &C1::vm2) ;
  sig2->connect (c2, &C2::m1) ;
  sig2->connect (c2, &C2::m2) ;
  sig2->connect (c2, &C2::vm1) ;
  sig2->connect (c2, &C2::vm2) ;
  std::cout << "--- Emitting sig2 #7...\n" ;
  sig2->emit("e7_0", "e7_1", "e7_2") ;
  std::cout << "--- Emitting sig2 #8 with accumulator...\n" ;
  accum.str = "" ;
  sig2->emit(accum, "e8_0", "e8_1", "e8_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Disconnecting c1->m1, c1->vm2, c2->m2 and c2->vm1 from sig2...\n" ;
  sig2->disconnect(global_slot) ;
  sig2->disconnect(c1, &C1::m1) ;
  sig2->disconnect(c1, &C1::vm2) ;
  sig2->disconnect(c2, &C2::m2) ;
  sig2->disconnect(c2, &C2::vm1) ;
  std::cout << "--- Emitting sig1 #9...\n" ;
  sig1->emit("e9_0", "e9_1", "e9_2") ;
  std::cout << "--- Emitting sig1 #10 with accumulator...\n" ;
  accum.str = "" ;
  sig1->emit(accum, "e10_0", "e10_1", "e10_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #11...\n" ;
  sig2->emit("e11_0", "e11_1", "e11_2") ;
  std::cout << "--- Emitting sig2 #12 with accumulator...\n" ;
  accum.str = "" ;
  sig2->emit(accum, "e12_0", "e12_1", "e12_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Deleting c1...\n" ;
  delete c1 ;
  c1 = 0 ;
  std::cout << "--- Emitting sig1 #13...\n" ;
  sig1->emit("e13_0", "e13_1", "e13_2") ;
  std::cout << "--- Emitting sig1 #14 with accumulator...\n" ;
  accum.str = "" ;
  sig1->emit(accum, "e14_0", "e14_1", "e14_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #15...\n" ;
  sig2->emit("e15_0", "e15_1", "e15_2") ;
  std::cout << "--- Emitting sig2 #16 with accumulator...\n" ;
  accum.str = "" ;
  sig2->emit(accum, "e16_0", "e16_1", "e16_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Deleting sig1...\n" ;
  delete sig1 ;
  sig1 = 0 ;
  std::cout << "--- Emitting sig2 #17...\n" ;
  sig2->emit("e17_0", "e17_1", "e17_2") ;
  std::cout << "--- Emitting sig2 #18 with accumulator...\n" ;
  accum.str = "" ;
  sig2->emit(accum, "e18_0", "e18_1", "e18_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Deleting c2...\n" ;
  delete c2 ;
  std::cout << "--- Emitting sig2 #19...\n" ;
  sig2->emit("e19_0", "e19_1", "e19_2") ;
  std::cout << "--- Emitting sig2 #20 with accumulator...\n" ;
  accum.str = "" ;
  sig2->emit(accum, "e20_0", "e20_1", "e20_2") ;
  std::cout << "accum.str = " << accum.str << "\n" ;

  std::cout << "--- Deleting sig2...\n" ;
  delete sig2 ;

  std::cout << "--- Test program END ----\n" ;
  return 0 ;
}
